unit DXGUI;
// allows for a gui on a DelphiX surface
interface
uses windows,Messages,Classes,Graphics,Controls,DXDraws,DXInput,DXmisc;
// This prevents the proc 'ProccessInput' from actually doing anything
// As this proc causes ALL the DXcontrols to have there DoProccessInput called
//{$_DEFINE NoProccessInput}

{$DEFINE NOHACKcollapse}
type
  // Forward declarations
  TCustomDXControl = class;
  TCustomDXContainer = class;  
  TDXDrawGUI = class;
  
  // Events Types
  TDXDrawEvent     = Procedure (Sender : TObject;
                                var Surface : TDirectDrawSurface;
                                var OldSurface : TDirectDrawSurface;
                                var Modified : boolean) of object;

  TCustomDXControl = class(TGraphicControl)
   Private
     { Private declarations }
     // Data
     // Pointer to the base of the GUI
     fDXDrawGUI : TDXDrawGUI;
     // these are the biuld-in support for image lists
     fImages : TCustomDXImageList;
     FPrimaryImage : TPictureCollectionItem;
     fImageIndex : integer;
     fCurrentPatternIndex : integer;
     // used to track when to update fOldSurface
     fModified : boolean;
     // this is the actual surface that is rendered to the screen
     fsurface : TDirectDrawSurface;
     // This surface is used to detect changes from the last frame
     fOldSurface : TDirectDrawSurface;
     // used to implement Transparent images
     fRawTransparentColor : longint;
     fTransparentColor : TColor;
     // tracks if the mouse is in the control
     fMouseInControl : boolean;
     fDXContainer : TCustomDXContainer;
     // options
     fHaveFocus : boolean;
     fTransparent : boolean;
     // Events
     fOnDraw : TDXDrawEvent;
     fOnKeyUp : TKeyEvent;
     fOnKeyDown : TKeyEvent;
     fOnKeyPress : TKeyPressEvent;
     fOnProccessInput : TNotifyEvent;
     fOnExit : TNotifyEvent;
     fOnEnter : TNotifyEvent;
     fOnMouseLeave: TNotifyEvent;
     FOnMouseEnter: TNotifyEvent;
   Protected
     { Protected declarations }
     fZlevel : integer;
     // This allows complex shaped images
     procedure CMHitTest(var Message: TCMHitTest); message CM_HITTEST;
     // These allow the OnMouseEnter & OnMouseExit events to work
     procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
     procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
          
     // This detects when we are placed on a TcustomDXContainer and correctly sets the parent property
     Procedure SetParent(Aparent : TWinControl); override;
//     procedure WndProc(var Message: TMessage); override;
     // Hooks when a TDXImagelist & TDXcontainer component is removed from the form
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;
     // makes sure that the graphics are update when it become visible
     procedure VisibleChanging; override;

     // This allows the component to initialize itself
     Procedure Loaded; override;
     
     Procedure OnDXInitialize; virtual;
     Procedure OnDXFinalize; virtual;

     // Adds the ability for a DXcontrol to grab focus on a click
     Procedure Click; override;
     // Updates the internal Z order for dx
     procedure SetZOrder(TopMost: Boolean); override;

     // Triggers the OnProccessInput event
     Procedure DoProccessInput; Dynamic;
     // Triggered by DXDrawGUI when the DXcontrol loses focus
     procedure DoExit; Dynamic;
     // Triggered by DXDrawGUI when the DXcontrol gains focus
     procedure DoEnter; Dynamic;
     
     Procedure DoKeyDown(var Key: Word; Shift: TShiftState); Dynamic;
     Procedure DoKeyUp(var Key: Word; Shift: TShiftState); Dynamic;
     Procedure DoKeyPress(var Key: Char); Dynamic;
     Procedure DoMouseEnter; Dynamic;
     Procedure DoMouseLeave; Dynamic;

     Procedure SetDXDrawGUI(ADXDrawGUI  : TDXDrawGUI); virtual;

     // Sets whether the control has focus
     Procedure SetHaveFocus(value : boolean); virtual;
     // This adds us to the list of components in the DXDrawGUI
     Procedure SetDXContainer(Container : TCustomDXContainer); virtual;
     // Sets the image used to represent the TCustomDXControl on screen
     Procedure SetImages( Value : TCustomDXImageList ); virtual;
     // This sets the primary image, this is the one that is displayed
     Procedure SetPrimaryImage(Value : TPictureCollectionItem); virtual;
     // Allows the primary image to be get/set by its index
     Procedure SetPrimaryImageIndex(Index : integer);  virtual;
     // This sets the pattern index 
     Procedure SetCurrentPatternIndex(Index : integer); virtual;
     // This sets the internal Z level
     Procedure SetZLevel(value : integer); virtual;
     // This sets the transparent value and extracts the transparent colour
     // off the surface
     Procedure SetTransparent( Value : boolean ); virtual;
     Procedure SetTransparentColor( Value : Tcolor ); virtual;
     // This draws the object to a standard Tcanvas
     { any desendants MUST implement this if the paint method if overriden
     if (DXContainer <> nil) and DXContainer.Collapsed
       dont draw images
     }
     Procedure Paint; override;
     // These are placed in the Protected section to allow desendance to control which
     // Properties are shown
     property Canvas;

     // Events
     // Occurs when a control loses the input focus, the mosue event
     // mouse leave is also triggered
     Property OnExit : TNotifyEvent read fOnExit write fOnExit;
     // Occurs when a control receives the input focus.
     Property OnEnter : TNotifyEvent read fOnEnter write fOnEnter;
     // Mouse events
     Property OnMouseUp;
     Property OnMouseDown;
     Property OnMouseMove;
     Property OnDblClick;
     // This is the event used to trigger OnEnter & OnExit
     Property OnClick;
     // Triggered when the mouse enters/leaves the region defining the image 
     Property OnMouseEnter : TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
     Property OnMouseLeave : TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
     // Keyboard events
     Property OnKeyUp : TkeyEvent read fOnKeyUp write fOnKeyUp;
     Property OnKeyDown : TkeyEvent read fOnKeyDown write fOnKeyDown;
     Property OnKeyPress : TKeyPressEvent read fOnKeyPress write fOnKeyPress;
   Public
     { Public declarations }

     Constructor create(AOwner: TComponent); Override;
     Destructor destroy; Override;

     // This is the routine called to draw the DXcontrol at Run-Time
     // The paint method is used to draw the DXcontrol ad Design-Time
     Procedure Draw; virtual;

     // this translates the bounds to be relative to DXDraw
     procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); Override;
     // Properties - most of these would be published in descendant

     // Data
     // This overrides the images info
     property Transparent : boolean read fTransparent write SetTransparent default true;
     // This is the raw colour value for the transparent colour on the drawing surface 
     property RawTransparentColor : LongInt read fRawTransparentColor;
     property TransparentColor : TColor read fTransparentColor write SetTransparentColor;
     // The image used to represent the TCustomDXControl on screen
     Property Images : TCustomDXImageList read FImages write SetImages;
     // This allows direct manipulation of the image
     Property Image : TPictureCollectionItem read fPrimaryImage write SetPrimaryImage;
     Property PrimaryImageIndex : integer read fImageIndex write SetPrimaryImageIndex;
     // This is the Z level, it controls when the menu is drawn
     // the lower it is the earlier it is drawn thus not garentied to be
     // completly drawn
     Property ZLevel : integer read fZlevel write SetZLevel;

     // Options

     // Used to determine if the DXcontol can recieve input-
     Property Enabled;
     Property Visible;
     // Indicates whether the contol is has focus and is active
     Property HaveFocus : boolean read fHaveFocus write SetHaveFocus default false;
     // This is the section of the image to use. For static images leave it at 0
     // For images that have multipule frames change this in the OnBeforeDraw event
     Property CurrentPatternIndex : integer read fCurrentPatternIndex write SetCurrentPatternIndex;
     // Events

     // the draw event is fired after the current image has been copied to the working surface
     Property OnDraw : TDXDrawEvent read fOnDraw write fOnDraw;

     // This gives the DXcontrol the chance to proccess input when it doesnt have focus
     Property OnProccessInput : TNotifyEvent read fOnProccessInput write fOnProccessInput;
     // this is the surface that is used for collistion detection & the source surface for any
     // actual drawing
     Property WorkSurface : TDirectDrawSurface read fsurface;
     // set this when ever WorkSurface is used to directly modifiy
     // and these modification take place outsize the OnDraw event
     Property SurfaceModified : boolean read fModified write fModified;
     // tracks if the mouse is in the control
     Property MouseInControl : boolean read fMouseInControl;
   Published
     { Published declarations }
     // Controls and owns the DXcontrol
     Property DXContainer : TCustomDXContainer read fDXContainer write SetDXContainer;
     Property DXDrawGUI  : TDXDrawGUI read fDXDrawGUI write SetDXDrawGUI;
   end; {TCustomDXControl}

  // This is the object which manages a group of DXcontrols
  TCustomDXContainer = class(TCustomDXControl)
   Private
     { Private declarations }
     // Data
     {$IFNDEF NOHACKcollapse}
     fNonCollapsedLocation : Trect;
     {$ENDIF}
     // Used to keep track of how far to move the client controls when the conatiner is moved 
     OldLeft,OldTop : integer;
     // List of all controls that are linked to this container
     DXControlList : TList;
     // Parts of the delphiX sprite Z level handling software have been copied
     // To provide z-level handling support
     DXDrawlist: TList;
     // options
     fActiveDXcontrol : TCustomDXControl;
     {$IFNDEF NOHACKcollapse}
     fCollapsed : boolean;
     {$ENDIF}
     // Events
   Protected
     { Private declarations }
     // This is used to detect when a non-DXcontrol is places on this container
     procedure ValidateInsert(AComponent: TComponent); Override;
     Procedure Loaded; Override;
     // This forces the parent property of 'obj' to the parent of the entire gui system
     Procedure SetDXControlParent(Obj : TCustomDXControl); virtual;
     // This allows us to detect when the parent control has changed
     // this then updates all the owned controls
     Procedure SetParent(AParent: TWinControl); override;

     // handle informing th attached DXControls of a change in the
     // DirectX object
     Procedure OnDXInitialize; override;
     Procedure OnDXFinalize; override;

     Procedure SetDXDrawGUI(ADXDrawGUI  : TDXDrawGUI); override;
     {$IFNDEF NOHACKcollapse}
     Procedure SetCollapsed(ACollapsed : boolean); virtual;
     {$ENDIF}

     // Sets the active DXcontrol, this is the control which is drawn to the forground and recieves input events from DXDrawGUI
     Procedure SetActiveDXcontrol(DXcontrol : TCustomDXControl); virtual;
     // Gets the number of DXcontrol that are linked to the screen
     Function GetDXControlCount : integer; virtual;
     // allows the management of the DXcontrols
     Function GetDXcontrol(index : integer) :  TCustomDXControl; virtual;
     // This is called internally by both TDXDrawGUI.LinkDXControl and
     // TCustomDXControl.SetZLevel
     // This is the method which interfaces with the Z level list for the DXcontrols
     // This method does sets the DXcontrol's Zlevel thriugh writing to the protected field 'Zlevel'
     // This method doesnt check if the is vaild
     Procedure SetDXControlZLevel(DXControl : TCustomDXControl;NewZlevel : integer); virtual;

     // These Procedures propergate messages/events into our menu system

     // By overriding these the container class can provide consistant
     // effects from one level of the gui to another level
     // This occurs when the active control in the container loses focus
     Procedure Click; override;
     // This occurs when the container loses focus, thus the active control loses focus
     procedure DoExit; override;


     // This allows for input and draw events to be forwarded along the gui
     Procedure DoProccessInput; override;
     Procedure DoKeyDown(var Key: Word; Shift: TShiftState); override;
     Procedure DoKeyUp(var Key: Word; Shift: TShiftState); override;
     Procedure DoKeyPress(var Key: Char); override;
   Public
     { Private declarations }
     Constructor Create(Aowner : Tcomponent); override;
     Destructor Destroy; override;

     // Draws the TcustomDXcontainer
     Procedure Draw; override;

     // This allows the DXcontrols that are 'owned' by this container
     // to be moved when we move
     procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); Override;
     
     // processes the input
     // Grabs input from DXInput and mouse input from MousePos
     Procedure ProccessInput;
     // Draws all the visible menus eta to the DX surface
     Procedure DrawAll;
     // This clears the internal lists, ie drawlist & DXcontrol list
     // But it doesnt destroy any of the objects
     // That is managed by the streaming system
     Procedure Clear;
     // Allows for management of all linked DXcontrols

     // This allows the TCustomDXControl to link into the container
     Procedure UnlinkDXControl(Obj : TCustomDXControl);
     // This changes the TCustomDXControl's owner
     Procedure LinkDXControl(Obj : TCustomDXControl);

     procedure TopMost(DXcontrol : TCustomDXControl);
     procedure BottomMost(DXcontrol : TCustomDXControl);
     // Transverses the list of TCustomDXContainer searching for
     // the primary container (the conatiner has a nil 'DXContainer' Property)
     function GetPrimaryDXConatiner : TCustomDXContainer;
     // Allows access to the list of owned controls
     Property DXcontrols[index : integer ] : TCustomDXControl read GetDXcontrol; default;
     Property DXControlCount : integer read GetDXControlCount;
   Published
     { Private declarations }
     // The DXcontrol that is active and the top most
     Property ActiveDXcontrol : TCustomDXControl read fActiveDXcontrol write SetActiveDXcontrol;
     Property Visible;
     {$IFNDEF NOHACKcollapse}
     // Collapse the menu tree under the container
     // this is effective in both run-time & design-time
     Property Collapsed : boolean read fCollapsed write SetCollapsed;
     {$ENDIF}
   end; {TCustomDXContainer}

  // PropergateInput -
  //   Forwards input events to the high level window when
  //   they are passed to DXDrawGUI
  // StickyActiveControl -
  //   Controls how input focus is received.
  //   When true, The active control remains active till the mouse is over
  //   another control.
  //   When false, the active control loses focus as soon as the mouse
  //   leaves the control
  // AllowDisabledDXcontrols -
  //   Controls how disabled DXcontrols obtain input focus
  //   When true disabled DXcontrols can recieve input focus
  TDXMenuOption = (PropergateInput,StickyActiveControl,AllowDisabledDXcontrols);
  TDXMenuOptions = set of TDXMenuOption;

  // This is required to give a foundation for the menus/buttons eta
  TDXDrawGUI = class(TDXDraw)
   Private
     { Private declarations }
     // Data
     fDXInput : TCustomDXInput;
     fMousePos : TMousePos;
     fDXControlContainer : TCustomDXContainer;
     // options
     fOptionsEx : TDXMenuOptions;
     // Events
   Protected
     { Protected declarations }
     // This allows us to detect when the parent control has changed
     Procedure SetParent(AParent: TWinControl); override;

     // This doesnt do anything
     Procedure SetDXControlContainer(ADXControlContainer : TCustomDXContainer); virtual;
     // Hooks the DXdraw object to signal various state changes on
     // the DDraw surface
     Procedure DXDrawNotifyEvent(Sender: TCustomDXDraw; NotifyType: TDXDrawNotifyType); virtual;
     // Manages the registering/unregistering of the DXDrawNotify event and other
     Procedure DXSetup(DXdraw : TCustomDXDraw); virtual;

     // Input events
     procedure Click; override;
     procedure KeyDown(var Key: Word; Shift: TShiftState); override;
     procedure KeyUp(var Key: Word; Shift: TShiftState); override;
     procedure KeyPress(var Key: Char); override;

  Public
     { Public declarations }
     Constructor create(AOwner: TComponent); Override;
     Destructor destroy; Override;

     // This automates to proccess of updating input
     Procedure UpdateInput;
     Procedure ProccessInputGUI;
     Procedure DrawGUI;

     // Gets the DXcontrol at the pos relative to the entire screen
     function GetDXControlAt(pos : Tpoint): TCustomDXControl; overload;
     function GetDXControlAt(x,y : integer): TCustomDXControl; overload;
     // this finds the DXcontrol which is active
     function GetActiveDXcontrol : TCustomDXControl;
     // Contains an object which handles obtain and converting the mouses postion
     Property MousePos : TMousePos read fMousePos;
     
   Published
     { Published declarations }
     Property OptionsEx : TDXMenuOptions read fOptionsEx write fOptionsEx;
     // This is the object which actually handles the DXcontrols
     Property DXControlContainer : TCustomDXContainer read fDXControlContainer write SetDXControlContainer;
     Property DXInput : TCustomDXInput read fDXInput write fDXInput;
   end; {TDXDrawGUI}

const
 // how the gui tree is collapsed is currently a hack that needs to be fixed
 // I'm open to suggestions
 CollapsedRect : Trect = (Left : -1000;Top : -1000; Right : -1000; Bottom : -1000);
 // This is the name of the primary root DXcontainer
 PrimaryContainer = 'Primary';
implementation
uses sysutils{,forms},DirectX,DXGUIClasses ;

// -----------------------------------------------------------------------------
// TCustomDXControl
// -----------------------------------------------------------------------------

Constructor TCustomDXControl.create(AOwner: TComponent);
begin
Inherited create(AOwner);
// extract the TDXDrawGUI & TCustomDXContainer classes
if AOwner is TCustomDXContainer then
  begin
  DXContainer := TCustomDXContainer(AOwner);
  DXDrawGUI := TCustomDXContainer(AOwner).DXDrawGUI;
  end
else if AOwner is TDXDrawGUI then
  begin
  DXDrawGUI := TDXDrawGUI(AOwner);
  DXContainer := TDXDrawGUI(AOwner).DXControlContainer;
  end;

if fDXDrawGUI <> nil then
  parent := fDXDrawGUI;
fTransparent := true;
Color := clbtnface;
canvas.Font.Color := clWhite;
canvas.Font.Size := 12;
//canvas.CopyMode := cmSrcPaint;
fImageIndex := -1;
Height := 32;
Width  := 32;
Visible := true;
ControlStyle := ControlStyle + [csReplicatable];
end; {create}

Destructor TCustomDXControl.destroy;
begin
FImages := nil;
Inherited destroy;
// unlink from the DXcontainer AFTER the inherited Destructor is called
// as this preserves the DXControl tree, till the end nodes have been freed
DXContainer := nil;
end; {destroy}

Procedure TCustomDXControl.SetParent(Aparent : TWinControl);
var
 Control: TWinControl;
 Pos : Tpoint;
begin
if TComponent(Aparent) is TCustomDXContainer then
  begin
  // Remap the parent Property
  if not (csdesigning in componentstate) then
    Control := TCustomDXContainer(Aparent).DXDrawGUI
  else
    // prevents stupid remaps when the DXDrawGUI property is set
    // at design time
    Control := nil;
  // make sure we arent re-mapping to a nil component  
  if Control = nil then
    Control := TCustomDXContainer(Aparent).parent;
  if (Control = nil) or (Control = parent) then
    exit;
  if (csdesigning in componentstate) then
    // Translate the posistion of the DXcontrol
    if Control.parent <> nil then
      begin
      Pos := point(Left,top);
      Pos := Control.ScreenToClient(Aparent.ClientToScreen(Pos));
      // move the control
      Left := Pos.x;
      Top  := pos.y;
      end;
  Aparent := Control;
  end;
if Aparent <> Parent then
  Inherited SetParent(Aparent);
end; {SetParent}

procedure TCustomDXControl.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
inherited Notification(AComponent, Operation);
if (Operation = opRemove) then
  begin
  if (FImages = AComponent) then
    // The image list is being removed, kill the referance to it
    Images := nil;
  if (DXDrawGUI = AComponent) then
    // The DXDrawGUI is being removed, kill the referance to it
    DXDrawGUI := nil;
  end;
end; {Notification}

procedure TCustomDXControl.VisibleChanging;
begin
Inherited VisibleChanging;
if visible then
  SurfaceModified := true;
end; {VisibleChanging}

Procedure TCustomDXControl.Loaded;
var
 NewIndex : integer;
 ADXDrawGUI : TDXDrawGUI;
begin
// We have been fully loaded
Inherited;
if fDXDrawGUI <> nil then
  begin
  // set the DXDrawGUI component
  ADXDrawGUI := fDXDrawGUI;
  fDXDrawGUI := nil;
  DXDrawGUI := ADXDrawGUI;
  end;
if assigned(fDXContainer) then
  begin
  NewIndex := fZlevel;
  // Link into the new Container
  fDXContainer.LinkDXControl(self);
  // set the Z level
  DXContainer.SetDXControlZLevel(self,NewIndex);
  end;
// Set the primary image
NewIndex := fImageIndex;
fImageIndex := -1;
SetPrimaryImageIndex(NewIndex);
// Set the tranparent colour
SetTransparent(Transparent);
end; {Loaded}

Procedure TCustomDXControl.OnDXInitialize;
begin
if not (csLoading In ComponentState) then
  begin
  // free the old surfaces
  FreeAndNil(fSurface);
  FreeAndNil(fOldSurface);
  // create the work surfaces
  fSurface    := TDirectDrawSurface.create(DXDrawGUI.DDraw);
  fOldSurface := TDirectDrawSurface.create(DXDrawGUI.DDraw);
  if (width * Height <> 0) then
    begin
    // now set the surface sizes
    fSurface.SetSize(Width,Height);
     foldSurface.SetSize(Width,Height);
    // clean the work surface
    if Transparent then
      begin // only bother IF it is Transparent
      // translate the TransparentColor to the surface RawTransparentColor
      SetTransparent(Transparent);
      // clean the work surface (with the transparent color)
      WorkSurface.Fill(RawTransparentColor);
      end;
    // make sure the graphics are updated
    SurfaceModified := true;
    end;
  end;
end; {OnDXInitialize}

Procedure TCustomDXControl.OnDXFinalize;
begin
// free the old surfaces
FreeAndNil(fSurface);
FreeAndNil(fOldSurface)
end; {OnDXFinalize}

Procedure TCustomDXControl.SetDXDrawGUI(ADXDrawGUI  : TDXDrawGUI);
begin
if (ADXDrawGUI <> fDXDrawGUI) then
  begin
  fDXDrawGUI := ADXDrawGUI;
  if not ((csDesigning in ComponentState) or
    (csLoading In ComponentState)) then
    begin
    if (fDXDrawGUI <> nil) and (fDXDrawGUI.DDraw <> nil) then
      OnDXInitialize
    else
      OnDXFinalize;
    end;
  end;
end; {SetDXDrawGUI}

Procedure TCustomDXControl.SetDXContainer(Container : TCustomDXContainer);
begin
if Container = self then
  exit;
if (csLoading In ComponentState) then
  begin
  // If we haven't fully loaded yet, Just set the value and exit
  fDXContainer := Container;
  exit;
  end;
if Container <> fDXContainer then
  begin
  // Unlink from the old Container
  if assigned(fDXContainer) then
    fDXContainer.UnlinkDXControl(self);
  fDXContainer := Container;
  // Link into the new Container
  if assigned(fDXContainer) then
    fDXContainer.LinkDXControl(self);
  end;
end; {SetDXContainer}

Function IsVisible(DXcontainer : TCustomDXcontainer) : boolean;
begin
// we trace the up the DXcontainer chain, checking for a in-visible
// container
while DXcontainer <> nil do
  begin
  if not DXcontainer.visible then
    begin
    result := false;
    exit;
    end;
  DXcontainer := DXcontainer.DXcontainer;
  end;
result := true;
end; {IsVisible}

procedure TCustomDXControl.CMHitTest(var Message: TCMHitTest);
begin
{$IFNDEF NOHACKcollapse}
if (DXContainer <> nil) and DXContainer.Collapsed then
  begin
  Message.Result := HTNOWHERE;
  exit;
  end;
{$ENDIF}
if IsVisible(DXcontainer) then
  begin
  if Transparent and (WorkSurface <> nil) then
    begin
    if WorkSurface.Pixels[Message.XPos,Message.YPos] <> RawTransparentColor  then
      Message.Result := HTCLIENT
    else
      Message.Result := HTNOWHERE;
    end
  else
    Inherited;
  end
else
  Message.Result := HTNOWHERE;
end; {CMHitTest}

procedure TCustomDXControl.CMMouseLeave(var Message: TMessage);
begin
inherited;
DoMouseLeave;
end; {CMMouseLeave}
                        
procedure TCustomDXControl.CMMouseEnter(var Message: TMessage);
begin
inherited;
DoMouseEnter;
end; {CMMouseEnter}

Procedure TCustomDXControl.Click;
begin
// We are now active
if assigned(DXContainer) then
  DXContainer.ActiveDXcontrol := self;
Inherited;
end;

procedure TCustomDXControl.SetZOrder(TopMost: Boolean);
begin
inherited SetZOrder(TopMost);
if assigned(DXContainer) then
if TopMost then
  DXContainer.TopMost(self)
else
  DXContainer.BottomMost(self)
end; {TCustomDXControl}

Procedure TCustomDXControl.DoMouseEnter;
begin
if Assigned(FOnMouseEnter) then
  fOnMouseEnter(Self);
fMouseInControl := true;
end; {DoMouseEnter}

Procedure TCustomDXControl.DoMouseLeave;
begin
if Assigned(FOnMouseLeave) then
  fOnMouseLeave(Self);
fMouseInControl := false;
end; {DoMouseLeave}

Procedure TCustomDXControl.DoKeyUp(var Key: Word; Shift: TShiftState);
begin
if assigned(fOnKeyUp) then
  fOnKeyUp(self,Key,Shift);
end; {DoKeyUp}

Procedure TCustomDXControl.DoKeyDown(var Key: Word; Shift: TShiftState);
begin
if assigned(fOnKeyDown) then
  fOnKeyDown(self,Key,Shift);
end; {DoKeyDown}

Procedure TCustomDXControl.DoKeyPress(var Key: Char);
begin
if assigned(fOnKeyPress) then
  fOnKeyPress(self,Key);
end; {DoKeyPress}

Procedure TCustomDXControl.DoProccessInput;
begin
{$IFNDEF NoProccessInput}
if assigned(fOnProccessInput) then
  fOnProccessInput(self);
{$ENDIF}
end; {DoProccessInput}

Procedure TCustomDXControl.DoExit;
begin
fHaveFocus := false;
if MouseIncontrol then
  DoMouseLeave;
if assigned(fOnExit) then
  fOnExit(self);
end; {DoExit}

procedure TCustomDXControl.DoEnter;
begin
// make sure we are visible
visible := true;
// activate all the containers above us, this will trigger the container to activate its container
if assigned(DXContainer) and assigned(DXContainer.DXContainer) then
  DXContainer.DXContainer.ActiveDXcontrol := DXContainer;
// we now have focus
fHaveFocus := true;
if assigned(fOnEnter) then
  fOnEnter(self);
end; {DoEnter}

Procedure TCustomDXControl.SetHaveFocus(value : boolean);
begin
if value <> fHaveFocus then
  begin
  fHaveFocus := value;
  if assigned(DXContainer) and not (csLoading In ComponentState) then
    begin
    if fHaveFocus then
      DXContainer.ActiveDXcontrol := self
    else
      DXContainer.ActiveDXcontrol := nil;
    end
  end;
end; {SetHaveFocus}

Procedure TCustomDXControl.SetPrimaryImageIndex(Index : integer);
begin
if (csLoading In ComponentState) then
  begin
  // If we haven't fully loaded yet, Just set the value and exit
  fImageIndex := Index;
  exit;
  end;
if assigned(FImages) then
  begin
  if (FImages.items.count <> 0) and
   (index < FImages.items.count) and
   (index >= 0) then
  Image := FImages.Items[Index];
  end;
end; {SetPrimaryImageIndex}

Procedure TCustomDXControl.SetCurrentPatternIndex(Index : integer);
begin
if (csLoading In ComponentState) then
  begin
  // If we haven't fully loaded yet, Just set the value and exit
  fCurrentPatternIndex := Index;
  exit;
  end;
if assigned(fPrimaryImage) then
  begin
  if (index >= 0) and
     (index < fPrimaryImage.PatternCount) then
  fCurrentPatternIndex := index;
  end;
end; {SetCurrentPatternIndex}

Procedure TCustomDXControl.SetPrimaryImage(Value : TPictureCollectionItem);
begin
if Value <> FPrimaryImage then
  begin
  FPrimaryImage := value;
  if (FPrimaryImage <> nil) and (FPrimaryImage.picture.Graphic <> nil) and
   not (FPrimaryImage.Picture.Graphic.Empty) then
    begin
    fImageIndex := FPrimaryImage.Index;

    Height := FPrimaryImage.Height;
    Width  := FPrimaryImage.Width;
    // important if we want to have the image being loaded
    fModified := true;
    // This allows images that need transperancy to be
    // properly blitted to the work surface and THEN to the actual
    // screen surface
    Transparent := FPrimaryImage.Transparent;
    if Transparent then
      TransparentColor := FPrimaryImage.TransparentColor;
    if (csDesigning in ComponentState) then
      begin
      Height := Height + 2;
      Width  := Width + 2;
      Invalidate;
      end
    end
  else
    begin
    fImageIndex := - 1;
    fCurrentPatternIndex := 0; 
    if (csDesigning in ComponentState) then
      begin
      // Set the default Height & width
      Height := 32;
      Width  := 32;
      Invalidate;
      end;
    end;
  end;
end; {SetPrimaryImage}

Procedure TCustomDXControl.SetImages( Value : TCustomDXImageList );
begin
if Value <> FImages then
  begin
  if assigned(FImages) then
    FImages.RemoveFreeNotification(self);
  FImages := Value;
  if (csLoading In ComponentState) then
    begin
    // If we haven't fully loaded yet, Just set the value and exit
    Exit;
    end;
  if assigned(FImages) then
    FImages.FreeNotification(self);
  // Try and reassign the primary image
  if assigned(FImages) and (FImages.Items.Count > 0)  then
    begin
    if (fImageIndex >= 0) and (fImageIndex < FImages.Items.Count) then
      SetPrimaryImageIndex(fImageIndex)
    else
      Image := FImages.Items[0];
    end
  else
    begin
    // clear the primary image
    Image := nil;
    end;
  end;
end; {SetDIB}

Procedure TCustomDXControl.SetTransparent( Value : boolean );
begin
fTransparent := Value;
SurfaceModified := true;
if (csLoading In ComponentState) or (csdesigning In ComponentState) then
  begin
  // If we haven't fully loaded yet, or we are in designing mode
  exit;
  end;
if fTransparent and (WorkSurface <> nil) then
  begin
  fRawTransparentColor := WorkSurface.ColorMatch(TransparentColor);
  WorkSurface.TransparentColor := fRawTransparentColor;
  end
else
  fRawTransparentColor := 0;
end; {SetTransparent}

Procedure TCustomDXControl.SetTransparentColor( Value : Tcolor );
begin
fTransparentColor := Value;
if Value = clNone then
  fTransparent := false; 
SetTransparent(Transparent);
end; {SetTransparentColor}

Procedure TCustomDXControl.SetZLevel(value : integer);
begin
if not (csLoading In ComponentState) then
  begin
  if (value <> fZlevel) and assigned(DXContainer) then
    DXContainer.SetDXControlZLevel(self,value);
  end
else
  fZlevel := value;
end; {SetZLevel}

Procedure TCustomDXControl.Paint;
begin
{$IFNDEF NOHACKcollapse}
if not((DXContainer <> nil) and DXContainer.Collapsed) then
{$ENDIF}
if (csDesigning in ComponentState) then
  begin
  // If we arent in designing mode then there is no point
  // to servicing the paint, as Ddraw is being used
  inherited Paint;
  with Canvas do
    begin

    if not assigned(FPrimaryImage) then
      begin
      // Try and obtain the primary image
      SetPrimaryImageIndex(0);
      if fImageIndex <> -1 then
        // exit as we have set another invalid event.
        // no point in drawing the image twice
        exit;
      end
    else if (FPrimaryImage.picture.Graphic <> nil) and
            (not FPrimaryImage.picture.Graphic.Empty) then
      Draw(0,0,FPrimaryImage.Picture.Graphic);
    Pen.Style := psDash;
    Brush.Style := bsClear;
    Rectangle(0,0,Width -1 ,Height - 1);
    end;
  end;
end; {DrawDesigner}

Procedure TCustomDXControl.Draw;
var
 oldTransparent : boolean;
begin
assert(fsurface <> nil,'Unable to draw self to a nil surface');
assert(fOldSurface <> nil,'Unable to draw to old surface');
assert(DXContainer <> nil,'Unable to access DXContainer');
assert(DXContainer.DXDrawGUI <> nil,'Unable to access DXContainer.DXDrawGUI');
if Height * width = 0 then
  exit;
// if the work surface has been modified, then store it to the old surface
if fModified then
  begin
  // store the Work Surface to the old surface
  fOldSurface.draw(0,0,rect(0,0,fsurface.Width,fsurface.Height),WorkSurface,false);
  if Transparent then
    begin // only bother IF it is Transparent
    // clean the work surface (with the transparent color)
    WorkSurface.Fill(RawTransparentColor);
    end;
  // copy the image to the work surface
  if assigned(FPrimaryImage) then
    begin
    // remove the Transparent flag, (allows the image to be blitted correctly)
    oldTransparent := FPrimaryImage.Transparent;
    FPrimaryImage.Transparent := false;
    FPrimaryImage.draw(WorkSurface,0,0,CurrentPatternIndex);
    // restore the old Transparent flag
    FPrimaryImage.Transparent := oldTransparent;
    end;
  // reset the modified flag
  fModified := false;
  end;
// Now let the user apply any effects to the surface before it is blitted
// to the actual sceen surface
if assigned(fOnDraw) then
  fOnDraw(self,fsurface,foldsurface,fModified);
{
more code here to handle that the primary work surface has been modified
}
// this actually draws the image
DXDrawGUI.surface.draw(left,top,WorkSurface,Transparent);
end; {Draw}

procedure TCustomDXControl.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
 oldWidth,oldHeight : Integer;
begin
oldWidth := Width;
oldHeight := Height;
Inherited SetBounds(ALeft,ATop,AWidth,AHeight);
if not (csDesigning in componentstate) and
   not (csloading in componentstate) and
   (oldWidth <> Width) or (oldHeight <> Height) then
  begin
  if (DXDrawGUI <> nil) and (DXDrawGUI.DDraw <> nil) and
     (DXDrawGUI.DDraw.IDDraw <> nil) then
    begin
    // resize the work surface, & old surface
    if (fSurface <> nil) then
      fSurface.SetSize(Width,Height);
    if (foldSurface <> nil) then
      foldSurface.SetSize(Width,Height);
    // changing the surface strips the color keys off the surface
    // thus removing the transperent blits
    SetTransparent(Transparent);
    end;  
  end;
end; {SetBounds}

// -----------------------------------------------------------------------------
// TCustomDXContainer
// -----------------------------------------------------------------------------

Constructor TCustomDXContainer.create(Aowner : Tcomponent);
begin
Inherited Create(Aowner);
// create ther various list used to keep track of info
DXControlList := Tlist.create;
DXDrawlist := Tlist.create;
// Force this component to accept controls
Controlstyle := Controlstyle + [csAcceptsControls];
end; {Create}

Destructor TCustomDXContainer.Destroy;
begin
// this unlinks the controls from the container
while DXControlList.count <> 0 do
  TCustomDXControl(DXControlList[DXControlList.count-1]).Free;
Inherited Destroy;
FreeAndNil(DXControlList);
FreeAndNil(DXDrawlist);
fDXDrawGUI := nil;
end; {Destroy}

// Overriden methods

procedure TCustomDXContainer.ValidateInsert(AComponent: TComponent);
begin
if not (AComponent is TCustomDXControl) then
  begin  // A non DXcontrol is being inserted
  // This makes sure its contruction is stopped
  Raise Exception.create('Atempting to place a non-DXcontrol on a DXcontrol-only container');
  end;
end; {ValidateInsert}

procedure TCustomDXContainer.Loaded;
begin
// We have been fully loaded
Inherited Loaded;
if assigned(fActiveDXcontrol) then
  begin
  // Set the active DXcontrol
  TopMost(fActiveDXcontrol);
  // fire off the OnEnter events
  fActiveDXcontrol.DoEnter;
  end;
{$IFNDEF NOHACKcollapse}
fCollapsed := not fCollapsed;
SetCollapsed(not fCollapsed);
{$ENDIF}
end; {Loaded}

Procedure TCustomDXContainer.OnDXInitialize;
var
  Index : integer;
begin
// this does the actual surface Initialization for this class
Inherited OnDXInitialize;
// now inform all the attached DXcontrols
if (DXControlList <> nil) then
  for index := 0 to DXControlList.count -1 do
    TCustomDxcontrol(DXControlList[index]).OnDXInitialize;
end; {OnDXInitialize}

Procedure TCustomDXContainer.OnDXFinalize;
var
  Index : integer;
begin
// this does the actual surface Finalization for this class
Inherited OnDXFinalize;
// now inform all the attached DXcontrols
if (DXControlList <> nil) then
  for index := 0 to DXControlList.count -1 do
    TCustomDxcontrol(DXControlList[index]).OnDXFinalize;
end; {OnDXFinalize}

function TCustomDXContainer.GetPrimaryDXConatiner : TCustomDXContainer;
var
  prevConatiner,PrimaryConatiner : TCustomDXContainer;
begin
assert(assigned(self),'Self = nil in TcustomDXcontainer.GetPrimaryDXConatiner');
PrimaryConatiner := self;
repeat
  prevConatiner := PrimaryConatiner;
  PrimaryConatiner := prevConatiner.DXContainer;
until not assigned(PrimaryConatiner);
result := prevConatiner; 
end; {GetPrimaryConatiner}

procedure TCustomDXContainer.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
 index : integer;
 DXControl : TCustomDXControl;
 Dpos : TPoint;
begin
if not (csLoading in ComponentState) then
  begin
  OldLeft := Left;
  OldTop := Top;
  // Allow the default handler to handle it
  Inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  DPos.x := Left - OldLeft;
  DPos.y := Top  - OldTop;
  // fixup all the owned controls positions
  if assigned(DXControlList) and
     not ((DPos.x = 0) and (DPos.y = 0)) then
    begin
    for index := 0 to DXControlList.Count - 1 do
      begin
      DXControl := TCustomDXControl(DXControlList.items[index]);

      DXControl.Left := DXControl.Left + DPos.x;
      DXControl.Top  := DXControl.Top + DPos.y;
      end;
    end;
  end
else
  begin
  Inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  end;
end; {SetBounds}


Procedure TCustomDXContainer.Click;
var
  DXControl : TCustomDXControl;
begin
with DXDrawGUI do
  begin
  MousePos.Update;
  // Update which Control is the active control
  DXControl := GetDXcontrolAt(MousePos.MouseX,mousepos.MouseY);
  if (DXControl = self) or
     (not assigned(DXControl)) and
     (not (StickyActiveControl in OptionsEx)) then
    self.SetActiveDXcontrol(nil)
  else
    self.SetActiveDXcontrol(DXControl);
  end;
Inherited;
end; {Click}

procedure TCustomDXContainer.DoExit;
begin
Inherited;
SetActiveDXcontrol(nil);
end; {DoExit}

Procedure TCustomDXContainer.DoProccessInput;
begin
{$IFNDEF NoProccessInput}
Inherited DoProccessInput;
ProccessInput;
{$ENDIF}
end; {DoProccessInput}

Procedure TCustomDXContainer.DoKeyDown(var Key: Word; Shift: TShiftState);
begin
Inherited;
if assigned(ActiveDXcontrol) then
  ActiveDXcontrol.DoKeyDown(key,shift);
end; {DoKeyDown}

Procedure TCustomDXContainer.DoKeyUp(var Key: Word; Shift: TShiftState);
begin
Inherited;
if assigned(ActiveDXcontrol) then
  ActiveDXcontrol.DoKeyUp(key,shift);
end; {DoKeyUp}

Procedure TCustomDXContainer.DoKeyPress(var Key: Char);
begin
Inherited;
if assigned(ActiveDXcontrol) then
  ActiveDXcontrol.DoKeyPress(key);
end; {DoKeyPress}

Procedure TCustomDXContainer.Draw;
begin
if not Visible then
  exit;
Inherited Draw;
DrawAll;
end; {Draw}

// Methods that manage the DXcontrols

Procedure TCustomDXContainer.Clear;
begin
// Since the streaming system destroys use AND the object we are referancing
// we dont have to free any of the DXcontrols
DXControlList.clear;
DXDrawlist.clear;
end; {Clear}

Procedure TCustomDXContainer.TopMost(DXcontrol : TCustomDXControl);
begin
if not assigned(DXcontrol) then
  exit;
DXDrawlist.Remove(DXcontrol);
DXDrawlist.Add(DXcontrol);
if DXDrawlist.count - 2 >= 0 then
  DXcontrol.fZlevel := TCustomDXControl(DXDrawlist[DXDrawlist.count - 2]).Zlevel + 1;
end; {TopMost}

procedure TCustomDXContainer.BottomMost(DXcontrol : TCustomDXControl);
begin
if not assigned(DXcontrol) then
  exit;
DXDrawlist.Remove(DXcontrol);
DXDrawlist.Insert(0,DXcontrol);
if DXDrawlist.count > 0 then
  DXcontrol.fZlevel := TCustomDXControl(DXDrawlist[0]).Zlevel + 1
else
  DXcontrol.fZlevel := 0;  
end; {BottomMost}

Procedure TCustomDXContainer.SetDXControlZLevel(DXcontrol : TCustomDXControl;NewZlevel : integer);
var
  L, H, I, C: Integer;
begin
DXcontrol.fZlevel := NewZlevel;
DXDrawlist.Remove(DXcontrol);
L := 0;
H := DXDrawlist.Count - 1;
while L <= H do
  begin
  I := (L + H) div 2;
  C := TCustomDXControl(DXDrawlist[I]).Zlevel - NewZlevel;
  if C < 0 then
    L := I + 1
  else
    H := I - 1;
  end;
DXDrawlist.Insert(L, DXcontrol);
end; {SetDXControlZLevel}

{$IFNDEF NOHACKcollapse}
Procedure TCustomDXContainer.SetCollapsed(ACollapsed : boolean);
begin
if (fCollapsed <> ACollapsed) and not (csLoading in ComponentState) then
  begin
  fCollapsed := ACollapsed;
  if (csDesigning in ComponentState) then
  if fCollapsed then
    begin
    // store the old position
    fNonCollapsedLocation := BoundsRect;
    // move the container of the page
    BoundsRect := CollapsedRect;
    end
  else
    begin
    // move the container back to its original spot
    BoundsRect := fNonCollapsedLocation;
    end;
  end;
end; {SetCollapsed}
{$ENDIF}

Procedure TCustomDXContainer.SetDXControlParent(Obj : TCustomDXControl);
begin
if (Obj.Parent <> DXDrawGUI) then
  begin
  //  Obj.Parent := TWinControl(DXDrawGUI);
  Obj.Parent := TWinControl(self);
  if (csdesigning in componentstate) then
    obj.Invalidate;
  end;
end; {SetDXControlParent}

Procedure TCustomDXContainer.SetParent(AParent: TWinControl);
var
 index : integer;
begin
Inherited SetParent(AParent);
if DXControlList <> nil then
for index := 0 to DXControlList.count -1 do
  TCustomDXControl(DXControlList[index]).Parent := Parent;
end; {SetParent}

Procedure TCustomDXContainer.SetDXDrawGUI(ADXDrawGUI  : TDXDrawGUI);
var
 Index : integer;
 oldDXDrawGUI : TDXDrawGUI;
begin
oldDXDrawGUI  := DXDrawGUI;
Inherited SetDXDrawGUI(ADXDrawGUI);
if (oldDXDrawGUI <> ADXDrawGUI) and
   (DXControlList <> nil) then
  begin
  for index := 0 to DXControlList.count -1  do
    TcustomDXControl(DXControlList[index]).DXDrawGUI := ADXDrawGUI;
  end;
end; {SetDXDrawGUI}

Procedure TCustomDXContainer.LinkDXControl(Obj : TCustomDXControl);
begin
if assigned(Obj) then
  begin
  if DXControlList.IndexOf(Obj) = -1 then
    begin
    // add the DXcontrol to the list
    DXControllist.Add(Obj);
    // add the DXcontrol to the z-order list
    DXDrawlist.Add(Obj);
    if DXDrawlist.count - 2 >= 0 then
      Obj.fZlevel := TCustomDXControl(DXDrawlist[DXDrawlist.count - 2]).Zlevel + 1;

    if (csDesigning in ComponentState) then
       begin // Desgin-Time specific code
       {$IFNDEF NOHACKcollapse}
       if fCollapsed then
         Obj.BoundsRect := rect(Obj.left - CollapsedRect.left,
                                Obj.Top - CollapsedRect.Top,
                                Obj.width - CollapsedRect.Right,
                                Obj.Height - CollapsedRect.Bottom);
       {$ENDIF}
       end;
    // force the DXControl 'Parent' property to this
    SetDXControlParent(Obj);
    // set the DXDrawGUI property
    if (Obj.DXDrawGUI <> DXDrawGUI) then
      begin
      if (DXDrawGUI = nil) then
        // the container doesnt have a DXDrawGUI property
        // grab the DXControls
        DXDrawGUI := Obj.DXDrawGUI
      else
        // override the current setting
        Obj.DXDrawGUI := DXDrawGUI;
      end;
    end;
  end;
end; {LinkDXControl}

Procedure TCustomDXContainer.UnlinkDXControl(Obj : TCustomDXControl);
var index : integer;
begin
if assigned(Obj) then
  begin
  Index := DXControlList.IndexOf(Obj);
  if index <> -1 then
    begin
    if ActiveDXcontrol = Obj then
      ActiveDXcontrol := nil;
    if (csDesigning in ComponentState) then
       begin // Desgin-Time specific code
       {$IFNDEF NOHACKcollapse}
       if fCollapsed then
         Obj.BoundsRect := rect(Obj.left + CollapsedRect.left,
                                Obj.Top + CollapsedRect.Top,
                                Obj.Width + CollapsedRect.Right,
                                Obj.Height + CollapsedRect.Bottom);
       {$ENDIF}                         
       end;
    (*
    // Snap the DXcontrols parent to the form
    Obj.Parent := self.Parent;
    // Make sure this shows up at design-time
    if assigned(self.Parent) and (csDesigning in ComponentState) then
      obj.Parent.Invalidate;
    *)
    // Remove the object form the draw list
    DXDrawlist.Remove(Obj);
     // Remove the object for the list of Dxcontrols
    DXControlList.Delete(Index);
    end;
  end;
end; {UnlinkDXControl}

Function TCustomDXContainer.GetDXControlCount : integer;
begin
result := DXControlList.Count;
end; {GetDXControlCount}

Function TCustomDXContainer.GetDXControl(index : integer) :  TCustomDXControl;
begin
if index < DXControlList.Count then
  result := TCustomDXControl(DXControlList[index])
else
  result := nil;  
end; {GetDXcontrol}

Procedure TCustomDXContainer.SetActiveDXcontrol(DXcontrol : TCustomDXControl);
var
 OldActiveDXcontrol : TCustomDXControl;
begin
if (csLoading In ComponentState) then
  begin
  // If we haven't fully loaded yet, Just set the value and exit
  fActiveDXcontrol := DXcontrol;
  Exit;
  end;
if DXcontrol <> fActiveDXcontrol then
  begin
  OldActiveDXcontrol := fActiveDXcontrol;
  fActiveDXcontrol := DXcontrol;
  TopMost(fActiveDXcontrol);
  // fire off the OnExit events
  if assigned(OldActiveDXcontrol) then
    OldActiveDXcontrol.DoExit;
  // fire off the OnEnter events
  if assigned(fActiveDXcontrol) then
    fActiveDXcontrol.DoEnter;
  end;
end; {SetActiveDXcontrol}

Procedure TCustomDXContainer.ProccessInput;
var
 index : integer;
begin
// call the input proccessing
for index := 0 to DXControlList.Count - 1 do
  TCustomDXControl(DXControlList.items[index]).DoProccessInput;
end; {ProccessInput}

Procedure TCustomDXContainer.DrawAll;
var                    
 index : integer;
begin
for index := 0 to DXDrawlist.Count - 1 do
 with TCustomDXControl(DXDrawlist.items[index]) do
   begin
   if visible {$IFNDEF NOHACKcollapse}and not Collapsed {$ENDIF} then
     Draw;
   end;
end; {DrawAll}
// -----------------------------------------------------------------------------
// TDXDrawGUI
// -----------------------------------------------------------------------------

Constructor TDXDrawGUI.create(AOwner: TComponent);
begin
Inherited create(AOwner);
fDXcontrolContainer := TCustomDXContainer.create(self);
DXControlContainer.Transparent := false;
DXControlContainer.DXDrawGUI := self;
DXControlContainer.parent := self;
DXControlContainer.Name := PrimaryContainer;
DXControlContainer.BoundsRect := rect(0,0,0,0);
// This allows us to accept controls at design time
ControlStyle := ControlStyle + [csAcceptsControls];
fOptionsEx := [PropergateInput];
// Set up the internal data
DXSetup(self);
end; {create}

Destructor TDXDrawGUI.destroy;
begin
DXSetup(nil);
Inherited destroy;
end; {destroy}

Procedure TDXDrawGUI.SetParent(AParent: TWinControl);
begin
Inherited SetParent(AParent);
if assigned(Parent) then
  begin
  // Create the object to track the mouses position change
  if not assigned(MousePos) then
    fMousePos := TMousePos.create(self);
  // Set the control the mouse is to clip to
  MousePos.Control := self;
  end
else
  begin
  // Free the mouse object
  MousePos.free;
  // The Destructor calls SetParent(nil) thus we can
  end;  
end;

procedure TDXDrawGUI.DXSetup(DXdraw : TCustomDXDraw);
begin
if self <> nil then
  self.UnRegisterNotifyEvent(DXDrawNotifyEvent);
if (DXdraw <> nil) and (DXdraw = self) then
  begin
  DXdraw.RegisterNotifyEvent(DXDrawNotifyEvent);
  end;
end; {DXSetUp}

Procedure TDXDrawGUI.SetDXControlContainer(ADXControlContainer : TCustomDXContainer);
begin
// dummy proc
end; {SetDXControlContainer}

function TDXDrawGUI.GetDXcontrolAt(pos: Tpoint): TCustomDXControl;
var
 temp : Tcontrol;
begin
// since all the loaded DXcontrols have the same parent, this will work
temp := DXControlContainer.parent.ControlAtPos(pos,AllowDisabledDXcontrols in OptionsEx,false);
if temp is TCustomDXControl then
  result := TCustomDXControl(temp)
else
  result := nil;
end; {GetDXcontrolAt}

function TDXDrawGUI.GetDXControlAt(x,y : integer): TCustomDXControl;
var
 temp : Tcontrol;
begin
// since all the loaded DXcontrols have the same parent, this will work
temp := DXControlContainer.parent.ControlAtPos(point(x,y),AllowDisabledDXcontrols in OptionsEx,false);
if temp is TCustomDXControl then
  result := TCustomDXControl(temp)
else
  result := nil;
end; {GetDXcontrolAt}

function TDXDrawGUI.GetActiveDXcontrol : TCustomDXControl;
var
 currentDXcontrol : TCustomDXContainer;
begin
result := nil;
currentDXcontrol := DXControlContainer;
// this transverse the ActiveDXcontrol linked list, to locate
// the current Active DXcontrol
while currentDXcontrol <> nil do
  begin
  if currentDXcontrol.ActiveDXcontrol is TCustomDXContainer then
    currentDXcontrol := TCustomDXContainer(currentDXcontrol.ActiveDXcontrol)
  else
    begin
    if currentDXcontrol.ActiveDXcontrol <> nil then
      result := currentDXcontrol.ActiveDXcontrol
    else
      result := currentDXcontrol;
    currentDXcontrol := nil;
    end;
  end;
end; {GetActiveDXcontrol}

Procedure RestoreWorkSurfaces(DXContainer : TCustomDXContainer);
var
  Index : integer;
  Dxcontrol : TCustomDxcontrol;
begin
if (DXContainer <> nil) and (DXContainer.DXControlList <> nil) then
  for index := 0 to DXContainer.DXControlList.count -1 do
    begin
    Dxcontrol := DXContainer.DXControlList[index];
    // the actual code that does to work
    DXcontrol.SurfaceModified := true;
    // keep searching for TCustomDXContainer s
    if DXcontrol is TCustomDXContainer then
      RestoreWorkSurfaces(TCustomDXContainer(DXcontrol));
    end;
end; {RestoreWorkSurfaces}

procedure TDXDrawGUI.DXDrawNotifyEvent(Sender: TCustomDXDraw;
  NotifyType: TDXDrawNotifyType);
begin
case NotifyType of
  dxntInitialize :
    begin
    if assigned(MousePos) then
    // Update the mouse info
    if doFullScreen in Options then
      // In fullscreen, no need to clip the co-ords
      MousePos.ClipToControl := false
    else
      // Not in fullscreen, clip the co-ords to the window
      MousePos.ClipToControl := true;
    // inform all the DXcontrols that there is a valid DirectX object
    DXControlContainer.OnDXInitialize;
    end;
  dxntFinalize :
    begin
    // inform all the DXcontrols that there isnt a valid DirectX object
    fDXControlContainer.OnDXFinalize;
    end;
  dxntRestore :
    begin
    // When the surfaces are invalidated, this component MUST
    // search ALL the connected DXcontrols and set the
    // SurfaceModified to true!
    // this allows the clear handling of restoring the images
    fDXControlContainer.SurfaceModified := true;
    RestoreWorkSurfaces(fDXControlContainer);
    end;
  end;
end; {DXDrawNotifyEvent}

procedure TDXDrawGUI.Click;
var
  DXControl : TCustomDXControl;
begin
Inherited;
MousePos.Update;
// Update which Control is the active control
DXControl := GetDXcontrolAt(Point(MousePos.MouseX,MousePos.MouseY));
if (not assigned(DXControl)) and
   (not (StickyActiveControl in OptionsEx)) then
  DXcontrolContainer.SetActiveDXcontrol(nil);
end; {Click}

procedure TDXDrawGUI.KeyUp(var Key: Word; Shift: TShiftState);
begin
with DXcontrolContainer do
if assigned(factiveDXcontrol) and (PropergateInput in OptionsEx) then
  activeDXcontrol.DoKeyUp(Key,Shift);
inherited;
end; {KeyUp}

procedure TDXDrawGUI.KeyDown(var Key: Word; Shift: TShiftState);
begin
with DXcontrolContainer do
if assigned(activeDXcontrol) and (PropergateInput in OptionsEx) then
  activeDXcontrol.DoKeyDown(Key,Shift);
inherited;
end; {DoKeyDown}
                           
Procedure TDXDrawGUI.KeyPress(var Key: Char);
begin
with DXcontrolContainer do
if assigned(activeDXcontrol) and (PropergateInput in OptionsEx) then
  activeDXcontrol.DoKeyPress(Key);
inherited;
end; {KeyPress}

Procedure TDXDrawGUI.UpdateInput;
begin
MousePos.Update;
if assigned(DXInput) then
  DXInput.Update;
end; {UpdateInput}

Procedure TDXDrawGUI.ProccessInputGUI;
begin
{$IFNDEF NoProccessInput}
DXControlContainer.ProccessInput;
{$ENDIF}
end; {ProccessInputGUI}

Procedure TDXDrawGUI.DrawGUI;
begin
if Initialized then
with DXcontrolContainer do
if visible {$IFNDEF NOHACKcollapse} and not Collapsed {$ENDIF} then
  DrawAll;
end; {DrawGUI}


end.
