unit DXGUIClasses;

interface
uses windows,messages,classes,controls,DXdraws,DXGUI,graphics,DXPowerFont;
{$DEFINE NOHACKcollapse}
Type
 TDXControl = class(TCustomDXControl)
   published
     // Data
     Property Action;
     Property DXContainer;
     Property Images;
     Property PrimaryImageIndex;
     Property ZLevel;
     // options
     Property Enabled;

     Property Visible;

     property Transparent;
     property TransparentColor;
     // Events

     Property OnDraw;

     Property OnEnter;
     Property OnExit;
     Property OnMouseEnter;
     Property OnMouseLeave;
     Property OnMouseDown;
     Property OnMouseUp;
     Property OnMouseMove;
     Property OnDblClick;
     Property OnClick;
     Property OnKeyUp;
     Property OnKeyDown;
     Property OnKeyPress;
     Property OnProccessInput;
    end; {TDXControl}

 TDXContainer = class(TCustomDXContainer)
   published
     // Data
     Property Action;
     Property DXContainer;
     Property Images;
     Property PrimaryImageIndex;
     Property CurrentPatternIndex;
     Property ZLevel;
     // options
     Property Enabled;

     property Transparent;
     property TransparentColor;     
     // Events

     Property OnDraw;

     Property OnEnter;
     Property OnExit;
     Property OnMouseEnter;
     Property OnMouseLeave;
     Property OnMouseDown;
     Property OnMouseUp;
     Property OnMouseMove;
     Property OnDblClick;
     Property OnClick;
     Property OnKeyUp;
     Property OnKeyDown;
     Property OnKeyPress;
     Property OnProccessInput;
    end; {TDXContainer}
    
 TDXPanel = class(TDXContainer)
   Private
     { Private declarations }
   Protected
     { Protected declarations }
   Public
     { Public declarations }
     Constructor Create(Aowner : Tcomponent);Override;
     Destructor Destroy; Override;
   published
     { published declarations }
  end; {TDXpanel}

 TDXImage = class(TDXControl)
   Private
     { Private declarations }
   Protected
     { Protected declarations }
   Public
     { Public declarations }
   published
     { published declarations }
  end; {TDXImage}

  TDXLabel = class(TDXControl)
   Private
     { Private declarations }
     fFont : TDXPowerFont;
     fWindowsFont : TFont;
     fAutoSize : boolean;
     fAlignment: TAlignment;
     // used to store the value that is actually written to the screen
     ftext : string;
     OldOnDraw : TDXDrawEvent;
     fUseWindowsFonts : boolean;
   Protected
     { Protected declarations }
     Procedure Paint; Override;
     Procedure Loaded; Override;
     Procedure Notification(AComponent: TComponent; Operation: TOperation); Override;
     // this is what actually handles the text clipping
     Procedure DoAutoSize; virtual;

     // calcualtes the X&Y pos
     function CalcYPos(TextHeight : integer) : integer;
     function CalcXPos(TextWidth : integer) : integer;

     Procedure SetText(var message : TWMSetText); message WM_SetText;
     // support for suing windows fonts
     Procedure SetUseWindowsFonts(value : boolean); virtual;
     Procedure SetWindowsFont(value : TFont); virtual;

     // this resonds to changes in the AutoSize state
     Procedure SetAutoSize(value: boolean); virtual;
     Procedure SetAlignment(AAlignment : TAlignment); virtual;
     Procedure SetFont(value: TDXPowerFont); virtual;
     // this actually draws the text to the surface
     Procedure DrawText(Sender : TObject;
                        var Surface : TDirectDrawSurface;
                        var OldSurface : TDirectDrawSurface;
                        var Modified : boolean);
   Public
     { Public declarations }
     constructor create(Aowner : Tcomponent); Override;
     destructor Destroy; Override;
     
     Procedure Draw; Override;
   published
     { published declarations }
     Property Text;
     Property AutoSize : boolean read fAutoSize write SetAutoSize;
     property Alignment: TAlignment read fAlignment write SetAlignment;
     Property Font : TDXPowerFont read fFont write SetFont;
     Property WindowsFont : TFont read fWindowsFont write SetWindowsFont;
     // use windows fonts instead of custom bitmapped fonts
     Property UseWindowsFonts : boolean read fUseWindowsFonts write SetUseWindowsFonts;
   end; {TDXLabel}
  (*
  TDXButtonControl = class(TCustomDXControl)// needs work
   Private // needs work
     { Private declarations }
     fChecked : boolean;
     fClicksDisabled : boolean;

   Protected
     { Protected declarations }
     Procedure Click; override;

     function GetChecked: Boolean; virtual;
     procedure SetChecked(Value: Boolean); virtual;

     Property Checked : boolean read GetChecked write SetChecked default False;
     Property ClicksDisabled : boolean read fClicksDisabled write fClicksDisabled;
   Public
     { Public declarations }
   published
     { published declarations }
   end; {TDXButtonControl}
   *)
   TDXButton = class(TDXLabel)
   Private
     { Private declarations }
   Protected
     { Protected declarations }
   Public
     { Public declarations }
   published
     { published declarations }
   end; {TDXButton}

implementation
uses Sysutils;

// -----------------------------------------------------------------------------
// TDXlabel
// -----------------------------------------------------------------------------

constructor TDXlabel.create(Aowner : Tcomponent);
begin
Inherited create(Aowner);
fAlignment := taLeftJustify;
fWindowsFont := TFont.create;
TransparentColor := clwhite;
fWindowsFont.Color := clblack;
end; {create}

destructor TDXlabel.Destroy;
begin
FreeAndNil(fWindowsFont);
Inherited Destroy;
end; {Destroy}

function TDXlabel.CalcXPos(TextWidth : integer) : integer;
begin
result := 2;
case Alignment of
  taLeftJustify  : ;
  taCenter       : result := (Width - TextWidth) div 2;
  taRightJustify : result := Width - TextWidth - 2;
  else Alignment := taLeftJustify;
end;
end; {CalcX}

function TDXlabel.CalcYPos(TextHeight : integer) : integer;
begin
if height <> TextHeight then
  result := (height div 2) - (TextHeight div 2)
else
  result := 0;
end; {CalcY}

Procedure TDXlabel.DrawText(Sender : TObject;
                        var Surface : TDirectDrawSurface;
                        var OldSurface : TDirectDrawSurface;
                        var Modified : boolean);
var
  X,Y : integer;
  oldtransparent : boolean;
  fontimage : TPictureCollectionItem;
  canvas : TDirectDrawSurfaceCanvas;
begin
// draw the text
if not fUseWindowsFonts then
  begin
  with Font do
    begin
    X := CalcXPos(TextWidth(ftext));
    Y := CalcYPos(TextHeight(ftext));
    // remove the Transparent flag, (allows the image to be blitted correctly)
    fontimage := DXImageList.Items[FontIndex];
    oldtransparent := fontimage.transparent;
    fontimage.transparent := false;
    // render the text to the work surface
    TextOut(WorkSurface,X,Y,ftext);
    // restore the old Transparent flag
    fontimage.transparent := oldtransparent;
    end;
  end
else
  begin
  canvas := surface.canvas;
  try
  with canvas do
    begin
    Font.Assign(fWindowsFont);
    brush.style := bsClear;
    X := TextWidth(ftext);
    y := TextHeight(ftext);
    TextOut(CalcXPos(X),CalcYPos(Y),ftext);
    end;
  finally
    canvas.release;
  end;
  end;
OnDraw := OldOnDraw;
if @OldOnDraw <> nil then
  OldOnDraw(sender,surface,oldsurface,modified);
end; {DrawText}

Procedure TDXlabel.Draw;
var
  FontTransparentColor : Tcolor;
begin
if not fUseWindowsFonts then
  begin
  if (ftext <> '') and
     (Font <> nil) and (Font.FontIndex <> -1) and
     (Font.DXImageList <> nil) and (Font.FontIndex < Font.DXImageList.items.count) then
    begin
    FontTransparentColor := fFont.DXImageList.items[fFont.FontIndex].TransparentColor;
    if TransparentColor <> FontTransparentColor then
      // the Transparent Color of the font has changed, this reset it to the correct value
      TransparentColor := FontTransparentColor;
    // this hack is used because I cant think of any better way to add effects
    // besides chain the ondraw events
    if SurfaceModified then
      OnDraw := DrawText;
    end
  end
else if SurfaceModified then
  OnDraw := DrawText;

Inherited draw;
end; {Draw}

Procedure TDXlabel.Paint;
var
  X,y : integer;
  fTextWidth,fTextHeight : integer;
begin
{$IFNDEF NOHACKcollapse}
if (DXContainer <> nil) and DXContainer.Collapsed then
  exit;
{$ENDIF}
Inherited Paint;
// draw the text
if not fUseWindowsFonts then
  begin
  if (ftext = '') or
     (Font = nil) or (Font.FontIndex = -1) or
     (Font.DXImageList = nil) or (Font.FontIndex >= Font.DXImageList.items.count) then
    exit;
  with Font do
    begin
    fTextWidth := TextWidth(ftext);
    fTextHeight := TextHeight(ftext);
    canvas.Font.Height := fTextHeight;
    end
  end
else
  begin
  canvas.Font.Assign(fWindowsFont);
  fTextWidth := canvas.TextWidth(ftext);
  fTextHeight := canvas.TextHeight(ftext);
  end;
// calc the relative X value
X := CalcXPos(fTextWidth);
Y := CalcYPos(fTextHeight);
// render the text to the sceen
if not Autosize then
  canvas.TextRect(ClientRect,X,Y,ftext)
else
  canvas.TextOut(X,Y,ftext);
end; {Paint}

Procedure TDXlabel.SetUseWindowsFonts(value : boolean);
begin
if fUseWindowsFonts <> value then
  begin
  fUseWindowsFonts := value;
  // update the text value
  ftext := Text;
  // Now handling clipping
  DoAutoSize;
  if (csdesigning in componentstate) then
    Invalidate;
  end;
end; {SetUseWindowsFonts}

Procedure TDXlabel.DoAutoSize;
var
  image : TPictureCollectionItem;
  fTextWidth,fTextHeight : integer;
begin
if (csloading in componentstate) then
  exit;
// check to see if the value is longer that the width
if not fUseWindowsFonts then
  begin
  if (Font = nil) or (Font.FontIndex = -1) or
     (Font.DXImageList = nil) or (Font.FontIndex >= Font.DXImageList.items.count) then
    exit;
  with Font do
    begin
    image := DXImageList.items[FontIndex];
    fTextHeight := abs(image.Height);
    fTextWidth := length(ftext) * Offset;
    end
  end
else
  begin
  canvas.font.assign(windowsfont);
  fTextHeight := canvas.TextHeight(ftext);
  fTextWidth := canvas.TextWidth(ftext);
  end;
if fTextWidth > width then
  begin
  if not Autosize then
    // truncate the text
    setlength(ftext,width div (fTextWidth div length(ftext)))
  else
    // expand the size of the box
    width := fTextWidth;
  end;
if (Height < fTextHeight) and AutoSize then
  Height := fTextHeight;
end; {DoAutoSize}

Procedure TDXlabel.SetText(var message : TWMSetText);
begin
if not (csloading in  componentstate) then
  begin
  // Obtain the new value
  ftext := string(message.Text);
  // Now handling clipping
  DoAutoSize;
  SurfaceModified := true;
  end;
// now set the value
Inherited;
if (csdesigning in componentstate) then
  Invalidate;
end; {SetText}

Procedure TDXlabel.SetAutoSize(value: boolean);
begin
if value <> Autosize then
  begin
  fAutosize := value;
  // update the text value
  ftext := Text;
  // Now handling clipping
  DoAutoSize;
  end
end; {SetAutoSize}

Procedure TDXlabel.SetWindowsFont(value : TFont);
begin
fWindowsFont.assign(value);
end; {SetWindowsFont}

Procedure TDXlabel.SetFont(value: TDXPowerFont);
begin
if (value <> fFont) then
  begin
  if fFont <> nil then
    fFont.RemoveFreeNotification(self);
  fFont := value;
  if fFont <> nil then
    begin
    fFont.FreeNotification(self);
    // set the Transparent flag
    Transparent := true;
    end;
  // update the text value
  ftext := Text;
  // Now handling clipping
  DoAutoSize;
  end;
end; {SetFont}

Procedure TDXlabel.SetAlignment(AAlignment : TAlignment);
begin
if AAlignment <> fAlignment then
  begin
  fAlignment := AAlignment;
  if csdesigning in componentstate then
    Invalidate;
  end;
end; {SetAlignment}

Procedure TDXlabel.Loaded;
begin
inherited Loaded;
if (Font <> nil) then
  // Make sure the Font object is fully ready
  Font.FontIndex := Font.FontIndex;
// update the text value
ftext := Text;
// Now handling clipping
DoAutoSize;
end; {Loaded}

Procedure TDXlabel.Notification(AComponent: TComponent; Operation: TOperation);
begin
Inherited Notification(AComponent,Operation);
if (Operation = opRemove) then
  begin
  if (fFont = AComponent) then
    Font := nil;
  end;
end; {Notification}
// -----------------------------------------------------------------------------
// TDXPanel
// -----------------------------------------------------------------------------

Constructor TDXPanel.Create(Aowner : Tcomponent);
begin
Inherited Create(Aowner);
end; {Create}

Destructor TDXPanel.Destroy;
begin
Inherited Destroy;
end; {Destroy}

// -----------------------------------------------------------------------------
// TDXButtonControl
// -----------------------------------------------------------------------------
(*
Procedure TDXButtonControl.Click;
begin
if not ClicksDisabled then
 Inherited;
end; {Click}

function TDXButtonControl.GetChecked: Boolean;
begin
result := fchecked;
end; {GetChecked}

procedure TDXButtonControl.SetChecked(Value: Boolean);
begin
fchecked := Value;
end; {SetChecked}
*)
// -----------------------------------------------------------------------------
//  TDXButton
// -----------------------------------------------------------------------------


end.
