unit DXprop;

interface
uses Classes,DXGUI,TypInfo,DsgnIntf;


type       {  TCustomDXContainerProperty  }
  TActiveDXControlProperty = class(TComponentProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

 TContainerProperty = class(TComponentProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function GetValue : string; override;
    procedure SetValue(const Value: string);  override;
    procedure GetProperties(Proc: TGetPropEditProc); override;
    // On double click it snaps the parent DXcontainer to the primary container
    procedure Edit; override;    
  end;
  
implementation
uses Sysutils,DXmisc;
// -----------------------------------------------------------------------------
// TActiveDXControlProperty
// -----------------------------------------------------------------------------

function TActiveDXControlProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paValueList] - [paReadOnly];
end;

procedure TActiveDXControlProperty.GetValues(Proc: TGetStrProc);
var
 index : integer;
begin {GetValues}
if GetComponent(0) is TCustomDXContainer then
  begin
  with TCustomDXContainer(GetComponent(0)) do
  for index := 0 to DXControlCount - 1 do
    begin
    Proc(DXcontrols[index].name);
    end;
  end;
end; {GetValues}

// -----------------------------------------------------------------------------
// TContainerProperty
// -----------------------------------------------------------------------------

function TContainerProperty.GetAttributes: TPropertyAttributes;
begin
Result := inherited GetAttributes + [paValueList] - [paReadOnly];
if not (GetComponent(0) is TCustomDXControl) then
 result := result + [paSubProperties];
end; {GetAttributes}

procedure TContainerProperty.GetValues(Proc: TGetStrProc);
begin
Proc(PrimaryContainer);
Inherited GetValues(Proc);
end; {GetValues}

procedure TContainerProperty.Edit;
begin
if GetName <> PrimaryContainer then
  begin
  SetValue(PrimaryContainer);
  end;
end; {Edit}

function TContainerProperty.GetValue : string;
begin
result := inherited GetValue;
if AnsiPos('.'+PrimaryContainer,result) > 0 then
  begin
  result := PrimaryContainer;
  end;
end; {GetValue}

procedure TContainerProperty.SetValue(const Value: string);

Function FindPrimaryContainer(DXcontrol : TCustomDXControl) : TCustomDXContainer;
var
 DXDrawGUI : TDXDrawGUI;
begin
if DXcontrol.Owner is TCustomDXContainer then
  begin
  result := TCustomDXContainer(DXcontrol.Owner).GetPrimaryDXConatiner;
  if not assigned(result) then
    result := FindPrimaryContainer(TCustomDXContainer(DXcontrol.Owner));
  exit;  
  end
else
  begin
  DXDrawGUI := TDXDrawGUI(FindComponentType(DXcontrol.Owner,TDXDrawGUI));
  if assigned(DXDrawGUI) then
    result := DXDrawGUI.DXControlContainer
  else
    result := nil;  
  exit;
  end;
result := nil;
end; {FindPrimaryContainer}

var
  DXcontrol : TCustomDXControl;
begin {SetValue}
if (TComponent(GetComponent(0)).name <> PrimaryContainer) then
if value = PrimaryContainer then
  begin
  if GetComponent(0) is TCustomDXControl then
    begin
    DXcontrol := TCustomDXControl(GetComponent(0));
    // find the object that contains the primary container
    // then set the DXcontainer prop
    DXcontrol.DXContainer := FindPrimaryContainer(DXcontrol);
    if not assigned(DXcontrol.DXContainer) then
      Raise Exception.create('Unable to find the Primary GUI container') at @TContainerProperty.SetValue;
    Modified;
    end
  else
    Raise Exception.create('Unable to handle setting the property : "DXcontainer"'#10'on a non-TCustomDXControl object') at @TContainerProperty.SetValue;
  end
else
  begin
  inherited SetValue(Value);
  end;
end; {SetValue}

procedure TContainerProperty.GetProperties(Proc: TGetPropEditProc);
var
 Components : TDesignerSelectionList;
begin
Components := TDesignerSelectionList.create;

Components.add(TCustomDXcontainer(FindComponentType(GetComponent(0) as Tcomponent,TCustomDXcontainer)));
GetComponentProperties(Components,[tkClass,tkEnumeration],Designer,Proc);
end; {GetProperties}

end.
